package Lento::ACT;
use strict;
use POE;
use Lento::Debuggable;
use Carp;

@Lento::ACT::ISA = qw(Lento::Debuggable);
$Lento::ACT::level = 1;

$ACT::last_ACT = -1;
$ACT::ACT_table = {};

=pod

=head1  NAME

Lento::ACT - Asynchronous Completion Tokens

=head1 SYNOPSIS

Lento::ACT->new($session);
my $session = Lento::ACT->token_to_session($token);
remove_ACT($token);


=head1 DESCRIPTION

Attach integers to sessions for packet dispatch.

=cut



# arg0 is class, arg1 is session
sub new ($$) {
    my $session = $_[1];

    $ACT::last_ACT++;
    $ACT::ACT_table->{$ACT::last_ACT} = $session;

    $poe_kernel->alias_set_for_session($ACT::last_ACT, $_[1]);

    DEBUG("new ACT $ACT::last_ACT for connection $session->{peer}\n");
    return $ACT::last_ACT;
}

# arg0 is class, arg1 is ACT
sub token_to_session($$) {
    return $ACT::ACT_table->{$_[1]};
}

# arg0 is class, arg1 is ACT
sub remove_ACT ($$) {
    my $session = $ACT::ACT_table->{$_[1]};
    if (defined $session) {

	$poe_kernel->alias_remove_for_session($_[1], $session);
	delete $ACT::ACT_table->{$_[1]};

	my $keys = keys %{$ACT::ACT_table};
	# why do I have to do -1 here?
	DEBUG("Removing ACT %d (%d ACTs remain in table)\n", $_[1], $keys);

    } else {
	confess(">>>> ***  >>>>> Trying to remove_ACT on token " . $_[1] .
		" which does not exist!\n");
    }
}

1;
