# building up a client cache
package Lento::Cache;
use strict;
use Lento::Debuggable;
use File::stat;
use POSIX qw(sys_stat_h errno_h);

@Lento::Cache::ISA = qw(Lento::Debuggable);
$Lento::Cache::level = -1;


# XXX fix me
my $SEEK_SET = 0;
#
# with strict enabled, $dirinfo->{$entry_name} barfs at "" for 
# $entry_name. Should fix this.
#
#use strict;


sub pack_dir($$) { 
    my $class = shift;
    my $path = shift;
    
    my $error = 0;
    my $buffer = "";

    DEBUG "Path: $path\n";

    my $st = stat($path);
    no strict 'refs';
    if (!$st) {
	$error = ENOENT;
    } elsif (!($st->mode() & 040000)) {
	# this file is not a directory
	$error = ENOTDIR;
    } elsif (!opendir($path, $path)) {
	# access denied, or similar
	$error = EPERM;
    } else {
	# XXX I haven't decided if this is an acceptable way of doing
	# things.  In my opinion it is almost certainly _not_, but we
	# must definitely talk about the advantages and drawbacks of a
	# couple of other ideas that I have in mind.
	my $entry_name;
	my $dirinfo;
	
	while ($entry_name = readdir($path)) {
	    next if $entry_name eq "..";
	    DEBUG "adding to hash: ``" . $entry_name . "''\n";
	    $dirinfo->{$entry_name} = lstat($path . "/" . $entry_name);
	}
	$buffer = Storable::nfreeze($dirinfo);
    }
    return ($error, $buffer);
}


sub fill_dir ($$$) {
    my $class = shift;
    my $path = shift;
    my $dirinfo = shift;
    my $entry;

    DEBUG "fill_dir on $path\n";

    foreach $entry (sort keys %{$dirinfo}) {
	my $st = $dirinfo->{$entry};

	my $entry_path = $path . "/" . $entry;
	$entry_path = $path if $entry eq ".";

	DEBUG "Handling $entry_path, mode " . $st->mode() . "\n";

	if ($st->mode() & 040000) {
	    Lento::Cache->install_dir($path, $entry, $st);
	} else {
	    Lento::Cache->install_file($path, $entry, $st);
	}

	my $stat = stat $entry_path || die "cannot stat new file";
    }
}

sub install_file ($$$) {
    my $class = shift;
    my $path = shift;
    die if (!defined $path);
    my $file = shift || die;
    my $stat = shift || die;
    my $cachepath = "$path/$file";

    DEBUG "class $class, path $path, cache path $cachepath\n";

    if (-e $cachepath && !-f $cachepath) {
	die "file $cachepath exists and is not a file";
    } else {
	my $st = stat($path) || die;
	DEBUG("install_file: parent directory time is " . gmtime($st->mtime) .
	      "\n");

	# set the length
	open FILE, "> $cachepath" || die "cannot create $cachepath";
	if ( $stat->size ) {
	    seek(FILE, $stat->size -1, $SEEK_SET) 
	      || die "cannot seek $cachepath: $!";;
	    print FILE "0";
	} 
	close FILE;

	chmod $stat->mode,  $cachepath || die "Cannot chmod\n";
	chown $stat->uid, $stat->gid, $cachepath || die "Cannot chmod\n";
	utime $stat->atime, $stat->mtime, $cachepath || die "Cannot utime\n";

	# reset parent directory time
	utime $st->atime, $st->mtime, $path || die "Cannot utime parent\n";
    }
    return 1;
}

sub install_dir ($$$) {
    my $class = shift;
    my $path = shift;
    die if (!defined $path);
    my $file = shift || die;
    my $stat = shift || die;
    my $cachepath = "$path/$file";

    die unless $::root;
    die unless $cachepath;

    my $st = stat($path) || die;
    DEBUG("install_dir: parent directory time is " . gmtime($st->mtime) .
	  "\n");

    if (-e $cachepath) {
	if (!-d $cachepath) {
	    DEBUG "ERROR: directory $cachepath exists and is not a directory";
	    die;
	}
    } else {
	DEBUG "install_dir: creating $cachepath\n";
  
	mkdir $cachepath, $stat->mode || die "Cannot mkdir\n";; 
    }

    chmod $stat->mode,  $cachepath || die "Cannot chmod\n";
    chown $stat->uid, $stat->gid, $cachepath || die "Cannot chown\n";
    utime $stat->atime, $stat->mtime, $cachepath || die "Cannot utime\n";

    # reset parent directory time
    utime $st->atime, $st->mtime, $path || die "Cannot utime\n";

    return 1;
}

1;

__END__

=head1 NAME

Cache - InterMezzo cache manager

=head1 SYNOPSIS
  # Use the cache
  use Cache;

  
  # install a file in the cache

  # $path represents the pathname of the file to be installed
  # $fileinfo is a hash:
  #    ->{statb} is a ref to a File::stat object
  #    ->{version} is the file version
  Cache::fill_file($path, $fileinfo);


  # install a directory in the cache

  # $path represents the pathname of the file to be installed
  # $fileinfo is a hash.  The keys are the names of entries in $path.
  # The values are anonymous hashes with:
  #    ->{statb} is a ref to a File::stat object
  #    ->{version} is the file version
  Cache::fill_dir($path, $dirinfo);

=head1 Description

  Cache installs files in the cache.  It presently depends on the $::root
  variable, which is the root of the cache.  

  Two methods are exported from Cache: fill_file which installs the
  contents of a file, and updates the version of the fso corresponding
  to the file.  Fill_dir does so similarly for directories.

  The Cache module is subordinate to the Fsdb - only the Fsdb should 
  call cache methods.

=head1 Uses

Fso: to update the versions
 

=head1 Bugs

Should have an instance, and instance variable $root. 

