package Lento::InterMezzo::UpcallHandler;
use POE;
use Lento::ReqDispatcher;
use Lento::Server;
use Lento::Filter::Upcall;
use Lento::Debuggable;
use Lento::Cache;
use POSIX qw(:errno_h :fcntl_h);

@Lento::Intermezzo::UpcallHandler::ISA = qw(Lento::Debuggable);
$Lento::InterMezzo::UpcallHandler::level = 1;

# these are not inside OpenFile because they're shared with
# InterMezzo::ReqHandler::Reintegrate
sub DAT {
    die "Protocol out of order" if (!$_[HEAP]->{handle});

    my $packet = $_[ARG0];
    $_[HEAP]->{handle}->write($packet->{body}, length($packet->{body}));
}

# ditto
sub EOD {
    die "Protocol out of order" if (!$_[HEAP]->{handle});

    my $packet = $_[ARG0];
    my $st = $_[HEAP]->{stat};

    $_[HEAP]->{handle}->close();
    delete $_[HEAP]->{handle};

    utime($st->mtime, $st->atime, $_[HEAP]->{path}) ||die"Cannot utime";
    $_[KERNEL]->yield('done', 0, $packet->{ctoken});

    $packet->{connection}->unuse();
}

sub OpenFile {
    my $request = shift;

    # can be an upcall or called from CML
    #
    # must have a path attribute and 4 methods listed below
    new POE::Session
      (
       [$request],  # one argument to _start
       $::dispatcher, ['connectme'],
       'Lento::InterMezzo::UpcallHandler', ['DAT', 'EOD'],
       $request, ['complete', 'got_connection', 'REP', 'done'],
       '_start' => sub {
	   $_[SESSION]->{name} = "OpenFile";
	   $_[SESSION]->option(trace => 1, default => 1) if
	     ($InterMezzo::UpcallHandler::level > 0);
	   $_[KERNEL]->alias_set($_[SESSION]);
	   my $request = $_[HEAP]->{completion} = $_[ARG0];

	   $request->{vol} = $::vldb->find_by_path($request->{path}) ||
	     die "no volume";

	   # connectme will ultimately result in got_connection or got_error
	   $_[KERNEL]->yield('connectme', $request);

	   DEBUG "OpenFile session = $_[SESSION]\n";
       },
       'got_error' => sub {
	   die "Fix me NOW";
       },
      );
}

sub OpenDir {
    my $upcall = shift;

    new POE::Session
      (
       [$upcall],  # one argument to _start
       $::dispatcher, ['connectme'],
       $upcall, ['complete'],
       '_start' => sub {
	   $_[SESSION]->{name} = "OpenDir ($_[ARG0]->{path})";
	   $_[SESSION]->option(trace => 1, default => 1) if
	     ($InterMezzo::UpcallHandler::level > 0);
	   $_[KERNEL]->alias_set($_[SESSION]);
	   my $upcall = $_[HEAP]->{upcall} = $_[ARG0];

	   $upcall->{vol} =
	     $::vldb->find_by_path($upcall->{path}) || die "no volume";

	   $_[KERNEL]->yield('connectme', $upcall);
       },
       'got_connection' => sub {
	   my $vol = $_[HEAP]->{upcall}->{vol};
	   my $conn = $vol->{server}->{connection};
	   $_[HEAP]->{path} = $vol->fs_path($_[HEAP]->{upcall}->{path});
	   $conn->sendpacket('REQ', ['SendDir', $vol->name(), $_[HEAP]->{path}]);
       },
       REP => sub {
	   my $packet = $_[ARG0];
	   my $rc = shift @{$packet->{parms}};
	   $_[HEAP]->{buffer} = "";

	   if ($rc != 0) {
	       $poe_kernel->yield('done', $rc, $packet->{ctoken});
	   }
       },
       DAT => sub {
	   if (!defined($_[HEAP]->{buffer})) {
	       die "Protocol out of order";
	   }
	   if ($_[HEAP]->{buffer}) {
	       die "Buffer already has data";
	   }

	   my $packet = $_[ARG0];
	   $_[HEAP]->{buffer} = $packet->{body};
       },
       EOD => sub {
	   if (!defined($_[HEAP]->{buffer})) {
	       die "Protocol out of order";
	   }

	   my $packet = $_[ARG0];
	   DEBUG "Buffer length: %d\n", length($_[HEAP]->{buffer});
	   my $buf = Storable::thaw($_[HEAP]->{buffer});
	   my $vol = $_[HEAP]->{upcall}->{vol};
	   Lento::Cache->fill_dir($vol->cache_path($_[HEAP]->{path}), $buf);
	   $_[KERNEL]->yield('done', 0, $packet->{ctoken});
	   $packet->{connection}->unuse();
       },
       'got_error' => sub {
	   die "Fix me NOW";
       },
       'done' => sub {
	   Lento::ACT->remove_ACT($_[ARG1]);
	   $_[HEAP]->{upcall}->complete($_[ARG0]);
	   $_[KERNEL]->yield('disconnectme', $_[HEAP]->{upcall});
	   $_[KERNEL]->alias_remove($_[SESSION]);
       }
      );
}

sub Open {
}

sub Signal {
}

sub Permit {
    my $upcall = shift;

    new POE::Session
      (
       [$upcall],  # one argument to _start
       $::dispatcher, ['connectme'],
       $upcall, ['complete'],
       '_start' => sub {
	   $_[SESSION]->{name} = "Permit ($_[ARG0]->{path})";
	   $_[SESSION]->option(trace => 1, default => 1) if
	     ($InterMezzo::UpcallHandler::level > 0);
	   $_[KERNEL]->alias_set($_[SESSION]);
	   $_[HEAP]->{upcall} = $_[ARG0];

	   $upcall->{vol} =
	     $::vldb->find_by_path($upcall->{path}) || die "no volume";
	   
	   if ( $vol->{have_permit} ) {
	       $upcall->complete(0);
	   } else {
	       $_[KERNEL]->yield('connectme', $upcall);
	   }
       },
       'got_connection' => sub {
	   my $vol = $_[HEAP]->{upcall}->{vol};
	   my $connection = $vol->{server}->{connection};
	   $connection->sendpacket('REQ', ['Permit', $vol->name()]);
       },
       REP => sub {
	   my $packet = $_[ARG0];
	   my $rc = shift @{$packet->{parms}};

	   $_[HEAP]->{upcall}->{vol}->{have_permit} = 1;
	   $packet->{connection}->unuse();
	   $poe_kernel->yield('done', $rc, $packet->{ctoken});
	   if ( $rc ) {
	       DEBUG "Permit returns error $rc\n";
	   }
       },
       'got_error' => sub {
	   die "Fix me NOW";
       },
       'done' => sub {
	   Lento::ACT->remove_ACT($_[ARG1]);
	   $_[HEAP]->{upcall}->complete($_[ARG0]);
	   $_[KERNEL]->alias_remove($_[SESSION]);
       }
      );
}


sub Journal {
    my $upcall = shift;

    new POE::Session
      (
       [$upcall], # one argument to _start
       '_start' => sub { 
	   $_[SESSION]->{name} = "Journal";
	   $_[SESSION]->option(trace => 1, default => 1) if
	     ($InterMezzo::UpcallHandler::level > 0);
	   $_[KERNEL]->alias_set($_[SESSION]);
	   $_[HEAP]->{upcall} = $_[ARG0];
	   $_[KERNEL]->yield('unpack');
	   DEBUG "Journal session = $_[SESSION]\n";
       },
       'unpack' => sub {
	   # Unpack places the records in the appropriate volume CMLs
	   Lento::Journal::Unpack($_[HEAP]->{upcall}->{buf});

	   my $volname;
	   foreach $volname (keys %{$::vldb->{vol_by_name}}) {
	       DEBUG "Dispatching $volname\n";
	       $::vldb->{vol_by_name}->{$volname}->{cml}->dispatch();
	   }

	   # the CML is responsible for the remainder of the reintegration
	   # process, so let this Upcall session die off
	   $_[KERNEL]->yield('done');
       },
       'done' => sub {
	   $_[KERNEL]->alias_remove($_[SESSION]);
       },
      );
}
