package Lento::List;
use Tie::RefHash;
use strict;

=pod

=head1 List

=head1 Description

List is a class to give the functionality of double linked lists.
The List object has one attribute which is an anonymous hash, Tie''d with RefHash to allow references to be stored in the List.

The list goes away when no references to it are held.

=head1 Methods

=over 4

=item B<new>: Instantiate a new list

=back

=cut

sub new {
    my $class = shift;
    my $self = {};
    $self->{list} = {};
    tie %{$self->{list}}, 'Tie::RefHash';
    bless $self;
}

=pod 

=over 4

=item B<add>: Add all arguments to the list.

=back

=cut

sub add {
    my $self = shift;
    
    foreach my $elt (@_) {
	if ( defined $self->{list}->{$elt} ) {
	    warn "Adding an existing element to $self";
	}
	$self->{list}->{$elt} = 1;
    }
}


=pod 

=over 4

=item B<iterate>: take a ref to a function, and other params. Call the
function for each member of the list.

=back

=cut

sub iterate {
    my $self = shift;
    my $reffun = shift;
    
    foreach my $session (keys %{$self->{list}}) {
	&$reffun($session, @_);
    }
}

=pod

=over 4

=item B<del>: Remove all arguments from the list

=back

=cut
sub del {
    my $self = shift;
    
    foreach my $elt (@_) {
	if ( ! defined $self->{list}->{$elt} ) {
	    warn "Removing a non-existing element $elt from $self";
	}
	delete $self->{list}->{$elt};
    }
}

=pod

=over 4 B<exists>

Takes one argument returns true if the element is in the list

=back

=cut

sub exists {
    my $self = shift;
    my $elt =shift;

    if ( defined ($self->{list}->{$elt} )) {
	 return 1;
     } else {
	 return undef;
     }
}

1;
