package Lento::Psdev;
use strict;
use IO::File;
use POSIX qw(:fcntl_h);
use Lento::Debuggable;
use POE;

@Lento::Psdev::ISA = qw(Lento::Debuggable);
$Lento::Psdev::level = -1;

#
# No object methods here
#

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
}

sub open {
    my $class = shift;
    my $whoami = shift;

    # guard against incomplete Perl installations, like mine
    die "Fcntl: O_RDWR not OK" unless O_RDWR == 2;

    $Lento::Psdev::handle = new IO::File("/dev/presto", O_RDWR);
    if (!$Lento::Psdev::handle) {
	die "Unable to open /dev/presto";
    }
    DEBUG "Opened /dev/presto, I'm pid $$\n";
    DEBUG "Ready to mount Intermezzo!\n";
    return 0;
}

#
# This is called if an EOF is ever received on this connection.  Since this is
# our link with the kernel, this is a Bad Thing(tm), and a fatal error.
#
sub cleanup {
    print "Aiee!  Lento received end-of-file on kernel link!\n";

    # If this function ever becomes non-fatal (!?), make sure to unregister()
    # the fd!

    die "Lento::Psdev: This is bad.  Email intermezzo\@inter-mezzo.org\n";
}


sub handle {
    return $Lento::Psdev::handle;
}

sub poll_journal ($) {
    my $self = shift;

    new POE::Session
      (
       [], # no arguments to start
       _start => sub {
	   $_[KERNEL]->yield('do_poll');
       },
       do_poll => sub {
	   Lento::Downcall->release();
	   $_[KERNEL]->delay('do_poll', 5);
       }
      );
}

sub read {
    my $package = shift;
    my $buf;
    my $rc = $Lento::Psdev::handle->sysread($buf, 4096);
    DEBUG "Lento::Psdev: Read $rc\n";
    return $buf;
}


sub write {
    my $package = shift;
    my $buf = shift;
    my $len = shift;
    DEBUG "Writing $len bytes of %d\n", length($buf);
    my $rc = $Lento::Psdev::handle->syswrite($buf, $len);
    DEBUG "Wrote $rc\n";

    if ($rc) {
	return $rc;
    } else {
	return -$Lento::Psdev::handle->error();
    }
}

1;
