package Lento::Vldb;
use strict;
use Lento::Debuggable;
use Lento::Volume;
use Lento::Server;

@Lento::Vldb::ISA = qw(Lento::Debuggable);
$Lento::Vldb::level = 0;

sub new ($) {
    my $class = shift;
    my $self = {};
    my $hash;
    my $buffer;    

    $self->{vol_by_mtpt} = {};
    $self->{vol_by_name} = {};

    LOG ">>>>> Initializing Vldb\n";
    open FH, "<voldb";
    while ( <FH> ) {
	$buffer .= $_;
    }
    close FH;

    $hash = eval ( $buffer );

    if (!defined($hash->{root})) {
	die "voldb must contain a \"root\" volume!";
    }
    if ($hash->{root}->{mtpt} ne "") {
	die "\"root\"'s mtpt must be \"\"!";
    }

    bless $self;

    my $volname;

    foreach $volname (keys %$hash) {
#	DEBUG "volname: $volname servername: $hash->{$volname}->{servername}\n";
	if (!$hash->{$volname}->{servername}) {
	    die "\"$volname\" has an invalid servername!";
	}
	if ($volname ne "root" && !$hash->{$volname}->{mtpt}) {
	    die "\"$volname\" has an invalid mtpt!";
	}

	my $vol = Lento::Volume->new($volname,
			      $hash->{$volname}->{servername},
			      $hash->{$volname}->{fshl},
			      $hash->{$volname}->{mtpt});
	$self->add($vol);
    }
    LOG "<<<<< Initializing Vldb\n";

    return $self;
}

# give it a path, return the volume object
sub find_by_path ($$) {
    my $self = shift;
    my $path = shift;

    while (!defined($self->{vol_by_mtpt}->{$path}) && $path) {
	my $foo = rindex($path, "/");
	if ($foo < 0) {
	    $path = "";
	} else {
	    $path = substr($path, 0, $foo);
	}
    }

    if ($path) {
	return $self->{vol_by_mtpt}->{$path};
    } else {
	return undef;
    }
}

sub find_by_name ($$) {
    my $self = shift;
    my $name = shift;

    return $self->{vol_by_name}->{$name};
}

sub add ($$) {
    my $self = shift; 
    my $vol = shift;

    LOG ">>    Added file set: %s:%s with mtpt %s\n", $vol->servername(),
	  $vol->fshl(), $vol->mtpt();

    $vol->server()->pushvol($vol);
    $self->{vol_by_name}->{$vol->name()} = $vol;
    $self->{vol_by_mtpt}->{$vol->mtpt()} = $vol;
}

sub printdb ($) {
    my $self = shift;
    my $result;
    my $name;

    foreach $name (keys %{$self->{vol_by_name}}) {
	$result .= $self->{vol_by_name}->{$name}->printme();
    }

    return $result;
}

1;
