package Lento::Volume;
use strict;
use Lento::CML;
use Lento::Connection;
use Lento::Debuggable;
use Lento::Server;

@Lento::Volume::ISA = qw(Lento::Debuggable);
$Lento::Volume::level = 0;

sub new ($$$$) {
    my $class = shift || die "No class";
    my $self = {};
    bless ($self, (ref($class) || $class));

    my $volname = $self->{name} = shift;
    $self->{servername} = shift;
    $self->{fshl} = shift;
    $self->{mtpt} = shift;
    die unless defined($self->{servername});
    die unless defined($self->{mtpt});

    my $server;
    if (!defined($server = $::serverdb->find_by_name($self->{servername}))) {
	$server = new Lento::Server($self->{servername}, $::server_port) || die;
	$::serverdb->add($server);
    }

    $self->{server} = $server;
    $self->{cml} = Lento::CML->new($self) || die;

    return $self;
}

sub name ($) {
    my $self = shift;

    return $self->{name};
}

sub mtpt ($) {
    my $self = shift;

    if ($self->{mtpt}) {
	return $::root . "/" . $self->{mtpt};
    } else {
	return $::root;
    }
}

sub servername ($) {
    my $self = shift;

    return $self->{servername};
}

sub server ($) {
    my $self = shift;

    return $self->{server};
}

sub fshl ($) {
    my $self = shift;

    return $self->{fshl};
}

#
# returns everything after volume root.  example:
# fs_path("/izo/foo") = "/foo"
#
sub fs_path ($$) {
    my $self = shift;
    my $path = shift;

    return substr($path, length($self->mtpt()));
}

#
# returns proper path for cache files, given relative path (fs_path).  example:
# cache_path("foo") = "/izo/foo"
#
sub cache_path ($$) {
    my $self = shift;
    my $path = shift;

    return $self->mtpt() . $path;
}

#
# returns proper path for storing files on server, given fs_path.  example:
# fshl_path("foo") = "/tmp/dvto/foo"
#
sub fshl_path ($$) {
    my $self = shift;
    my $path = shift;

    return $self->{fshl} . $path;
}

sub printme ($) {
    my $self = shift;

    return "Volume \"$self->{name}\" uses server \"$self->{servername}\" and "
      . "mtpt \"$self->{mtpt}\"\n";
}

1;
